---
layout: minimal
title: CSE 373
has_children: true
has_toc: true
seo:
  type: Course
  name: Data Structures and Algorithms
---

# {{ site.description }}
{: .mb-2 }
{{ site.tagline }}
{: .fs-6 .fw-300 }

This is CSE 373 Summer 2021 at the University of Washington. Looking for [**Autumn 2021**](https://courses.cs.washington.edu/courses/cse373/21au/)?
{: .code-example }

{% assign instructors = site.staffers | where: 'role', 'Instructor' %}
{% for staffer in instructors %}
{{ staffer }}
{% endfor %}

[Husky Maps](https://huskymaps.herokuapp.com/) is a web app for mapping the world, searching for places, and navigating around Seattle. All these features are powered by the sociotechnical infrastructure of **data structures and algorithms**, programming abstractions designed by software engineers to represent data and automate processes. CSE 373 asks the question: **How does the design of an abstraction affect implementations and clients?**

[Computing culture](https://youtu.be/c8TQ29I8lK4) has historically addressed this question from a [disembodied](https://en.wikipedia.org/wiki/Mind%E2%80%93body_dualism) technical perspective, viewing social perspectives on computing as less relevant than technical perspectives. But in order to address society's most challenging questions around the design and use of computing, we'll need to learn about the impacts of our programming in a deeply sociotechnical way---combining social and technical ideas. Just as machine learning and artificial intelligence is changing the world for better and for worse, the design of data structures and algorithms can raise equally important sociotechnical questions for the systems we build. In our first project, for example, we'll study *Autocomplete*, a feature of the Husky Maps search bar.

> Autocomplete is a feature that helps a user select valid search results by showing possible inputs as they type. Computer scientists have historically only asked questions such as, "Is this implementation of autocomplete correct?" or "How efficient is this implementation of autocomplete?" These are important questions, but they're not the only questions. In this course, we will ask these questions but also new questions such as, "How does this definition reshape civil and human rights?" and "What ideas about society does it encode?"

To begin to answer these complex sociotechnical questions, I believe we need to appreciate learning as an [embodied](https://en.wikipedia.org/wiki/Embodied_cognition) experience involving all of our humanity and ways of knowing.

Although CSE 373 is designed to be taken after [CSE 143](https://courses.cs.washington.edu/courses/cse143/)---we expect familiarity with array lists, linked lists, and search trees---the focus is not on programming but rather the **design, analysis, and critique** of the data structures and algorithms that implement software's social imaginations and designed futures.

1. Design data structures and algorithms by implementing and maintaining invariants.
1. Analyze the runtime and design values of data structures and algorithms.
1. Critique the application of data structures and algorithms toward social problems.

In the first 7 weeks, we'll study 3 interfaces and 6 applications of data structures and algorithms.

1. Autocomplete data structures and algorithms for **search suggestions** and **DNA indexing**.
1. Priority queue data structures for **content moderation** and **shortest paths**.
1. Graph data structures and shortest paths for **seam carving** and **navigation directions**.

In the final 2 weeks, we'll wrap up the course by reflecting on our journey through computing.

## Values and Policies

The education you receive in this course can help prepare you for programming jobs, but this isn't the only purpose for learning computer science.[^1] Education is not only about yourself and your personal gain, but also about all of us and our capacity to live together as a community.

[^1]: Mark Guzdial. 2021. [Computer science was always supposed to be taught to everyone, and it wasn't about getting a job](https://computinged.wordpress.com/2021/11/26/computer-science-was-always-supposed-to-be-taught-to-everyone-but-not-about-getting-a-job-a-historical-perspective/).

The University of Washington acknowledges the Coast Salish peoples of this land, the land which touches the shared waters of all tribes and bands within the Duwamish, Puyallup, Suquamish, Tulalip and Muckleshoot nations. Among the traditions of the Coast Salish peoples is a value for the connectedness between all living things and a recognition of the unique ways that each of us comes to know something.[^2]

[^2]: Roger Fernandes. 2012. [Roger Fernandes: Artist/Storyteller/Educator](https://youtu.be/O6sS1ZI8dDk).

> Modern education has the idea that we all need to know the same thing. At the end of the lesson, everyone will know the same thing. That's why we have tests, that's why we have quizzes, that's why we have homework: to ensure we all know the same thing. And that's powerful---that's important---within a certain context.
>
> But for native culture, the idea that each listener divines or finds their own answer, their own meaning, their own teaching from the story is equally powerful---that each person needs to be able to look at the world and define it for themselves within their culture and then also find a way to live in that world according to the teachings of their people in their culture.

Our course emphasizes the following values and policies.

We are responsible for each others' success
: Everyone has a right to feel like they belong in this course. We'll need to act with compassion and caring to collaborate with each other. Although we will need more than just unexamined commitments to collaboration, listening, empathy, mindfulness, and caring,[^3] the following guidelines offer a starting point for ensuring compassion toward each other.[^4]

  - Listen with intention to understand first and form an opinion only after you fully understand.
  - Take responsibility for the intended and unintended effects of your words and actions on others.
  - Mindfully respond to others' ideas by acknowledging the unique value of each contribution.

: You should expect and demand to be treated by your classmates and teachers with respect. If any incident occurs that challenges this commitment to a supportive, diverse, inclusive, and equitable environment, please let the instructor know so the issue can be addressed. Should you feel uncomfortable bringing up an issue with the instructor directly, meet our advisors during [quick questions](https://www.cs.washington.edu/academics/ugrad/advising#qqs) or contact the [College of Engineering](https://www.engr.washington.edu/bias).

[^3]: Brian Arao and Kristi Clemens. 2013. "From Safe Spaces to Brave Spaces: A New Way to Frame Dialogue Around Diversity and Social Justice" in *The Art of Effective Facilitation*.

[^4]: Asao B. Inoue. 2019. "Sample Charter for Compassion" in [Labor-Based Grading Contracts: Building Equity and Inclusion in the Compassionate Writing Classroom](https://wac.colostate.edu/books/perspectives/labor/).

We recognize everyone has unique circumstances
: Do not hesitate to contact the instructor by private discussion post or [appointment](https://kevinl.info/meet/). The sooner we are made aware of your circumstances, the more we can help. Extenuating circumstances include work-school balance, familial responsibilities, religious observations, military duties, unexpected travel, or anything else beyond your control that may negatively impact your performance in the class.
: It is the policy and practice of the University of Washington to create inclusive and accessible learning environments consistent with federal and state law. If you have already established accommodations with [Disability Resources for Students](https://depts.washington.edu/uwdrs/) (DRS), activate your accommodations via myDRS so we can discuss how they will be implemented in this course. If you have a temporary health condition or permanent disability that requires accommodations, contact DRS directly to set up an Access Plan.
: Washington state law requires that UW develop a policy for accommodation of student absences or significant hardship due to reasons of faith or conscience, or for organized religious activities. The UW's policy, including more information about how to request an accommodation, is available at [Religious Accommodations Policy](https://registrar.washington.edu/staffandfaculty/religious-accommodations-policy/). Accommodations must be requested within the first two weeks of this course using the [Religious Accommodations Request form](https://registrar.washington.edu/students/religious-accommodations-request/).

We believe everyone wants to learn
: Education is about shaping your identity as much as it is about learning things. In school, the consequences of making mistakes are relatively small. But the habits you form now---repeated over days, weeks, months, or years---determine who you will be in the future.
: We ask that you do not claim to be responsible for work that is not yours. When you receive substantial help from someone else, include a citation. Don't request a copy of someone else's work, don't provide your work to another student, and don't post your solutions publicly.
: Academic honesty reflects the trust (or the lack thereof) between students and teachers. We do our best to design the course in ways that ensure trust, but we know our systems are not perfect. If you submit work in violation of these policies but bring it to the attention of the instructor within 72 hours, you may resubmit your own work without further consequence. Rather than blame students, we want to fix or replace broken systems that compel students to lose trust.

## Deliberate practice

In a traditional classroom, you attend class while a teacher lectures until time is up. Then, you go home and do the important work of applying concepts toward practice problems or assignments on your own. Finally, you take an exam to show what you know.

This classroom structure isn't effective for learning science, engineering, and mathematics.[^5] Think of learning computer science as learning how to ride a bike. Quite a few people know how to ride a bike. But how many of them learned to ride a bike through 50 minutes of lecture three times a week? Probably no one---learning to ride a bike requires riding an actual bike! Likewise, learning computer science requires **deliberate practice**.

[^5]: Scott Freeman, Sarah L. Eddy, Miles McDonough, Michelle K. Smith, Nnadozie Okoroafor, Hannah Jordt, and Mary Pat Wenderoth. 2014. [Active learning increases student performance in science, engineering, and mathematics](https://doi.org/10.1073/pnas.1319030111).

1. Sustained motivation.
1. Tasks that build on prior knowledge.
1. Immediate, personalized feedback on tasks.
1. Repetition of all the above.

Consistent with our class values, learning is not only about content knowledge, but also about socioemotional motivation and support. Each week in the course will involve several different activities that are designed so that we can make the most of our class time together.

1. On your own before class, prepare for learning by completing the **pre-class preparation**.
1. In your team during class and quiz section, collaborate on the **in-class guided practice**.
1. On your own after quiz section, complete the **assessment** and submit an explanation video.
1. In your team throughout the module, apply your learning by collaborating on the **project**.

Expect to spend 4 hours in class and 8 hours outside of class working on this course. Depending on how you plan your project work, some weeks may be more work or less work than other weeks. If you find the workload is significantly exceeding this expectation, talk to your TA.

Grading in this course encourages learning through deliberate practice by emphasizing revision and resubmission of work. All coursework is designed around feedback loops where you try something, get feedback, then try again. Grades are based on what you eventually learn through this process. Only the requirements listed under a [Canvas]({{ site.canvas }}) module count toward your final grade.

1.0 or greater
: Completion of all requirements in the **Abstractions** module.
: Completion of all requirements in the **Autocomplete** module.

2.0 or greater
: Completion of all requirements in the **Abstractions** module.
: Completion of all requirements in the **Autocomplete** module.
: Completion of all requirements in the **Priority Queues** module.

3.0 or greater
: Completion of all requirements in the **Abstractions** module.
: Completion of all requirements in the **Autocomplete** module.
: Completion of all requirements in the **Priority Queues** module.
: Completion of all requirements in the **Shortest Paths** module.

4.0
: Completion of all requirements in the **Abstractions** module.
: Completion of all requirements in the **Autocomplete** module.
: Completion of all requirements in the **Priority Queues** module.
: Completion of all requirements in the **Shortest Paths** module.
: Highest marks across all parts of the **Portfolio**.

## References
